package org.numamo.services.rcm.data.adapter.service.v2x.impl;

import org.numamo.rcm.car.game.protocols.v2x.dto.ResponseStatus;
import org.numamo.rcm.car.game.protocols.v2x.dto.resume.GetResumeRq;
import org.numamo.rcm.car.game.protocols.v2x.dto.resume.GetResumeRs;
import org.numamo.rcm.car.game.protocols.v2x.dto.resume.info.GetResumeInfoRq;
import org.numamo.rcm.car.game.protocols.v2x.dto.resume.info.GetResumeInfoRs;
import org.numamo.rcm.car.game.protocols.v2x.resume.GameResumeV2x;
import org.numamo.services.rcm.data.adapter.service.v2x.api.ConfigHolder;
import org.numamo.services.rcm.data.adapter.service.v2x.api.RequestControl;
import org.numamo.services.rcm.data.adapter.service.v2x.api.ResumeStorageService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static java.util.Collections.emptyList;
import static java.util.Collections.unmodifiableList;
import static java.util.Objects.requireNonNull;


@Service
public final class ResumeStorageServiceImpl implements ResumeStorageService {


    // Variables and constructors:-------------------------------------------------------
    private static final Logger LOGGER = LoggerFactory.getLogger(ResumeStorageServiceImpl.class);

    private final ConfigHolder configHolder;
    private final RestTemplate restTemplate;
    private final RequestControl requestControl;

    @Autowired
    public ResumeStorageServiceImpl(ConfigHolder configHolder,
                                    RestTemplate restTemplate,
                                    RequestControl requestControl
    ) {
        this.configHolder = configHolder;
        this.restTemplate = restTemplate;
        this.requestControl = requestControl;
    }


    // Public API:-----------------------------------------------------------------------
    @Override
    public List<GameResumeV2x> getByKeyWords(final String keyWord,
                                             final String modifier
    ) {
        LOGGER.info("Requesting data server by the key={} and modifier={}", keyWord, modifier);

        final String url = configHolder.getSearchUrlWithKeyWordsV2x();
        final GetResumeInfoRq request = new GetResumeInfoRq();
        request.setKeyWord(keyWord);
        request.setModifier(modifier);
        request.setRequestId(requestControl.getCurrent().getRequestId());

        final Optional<GetResumeInfoRs> response = processStatus(
                restTemplate.postForEntity(url, request, GetResumeInfoRs.class));

        if (response.isPresent()) {
            final List<Long> tokens = new ArrayList<>();
            response.get().getResumeInfoList().forEach(i -> tokens.add(i.getToken()));
            return getByTokens(tokens);

        } else {
            return emptyList();
        }
    }

    @Override
    public List<GameResumeV2x> getByTokens(final List<Long> tokens) {

        LOGGER.info("Requesting game data by the following tokens: {}", tokens);

        final String url = configHolder.getSearchUrlWithTokensV2x();
        final GetResumeRq request = new GetResumeRq();
        request.setResumeTokens(tokens);
        request.setRequestId(requestControl.getCurrent().getRequestId());

        final Optional<GetResumeRs> responseOpt = processStatus(
                restTemplate.postForEntity(url, request, GetResumeRs.class)
        );

        if (responseOpt.isPresent()) {
            final GetResumeRs response = responseOpt.get();
            final ResponseStatus status = response.getStatus();
            if (status.isSuccess()) {
                return unmodifiableList(response.getGameResumes());
            } else {
                LOGGER.error("Requested services returned error status = {}", status);
            }
        }

        return emptyList();
    }


    // Internal methods:-----------------------------------------------------------------
    private <T> Optional<T> processStatus(ResponseEntity<T> response) {

        if (response.getStatusCode() == HttpStatus.OK) {
            final T body = requireNonNull(response.getBody(), "HTTP request body was set as NULL");
            LOGGER.trace("Requested service returned {}", body);
            return Optional.of(body);

        } else {
            LOGGER.debug("Requested services returned error status = {}", response.getStatusCode());
            return Optional.empty();
        }
    }

}
